@@ -56,10 +56,29 @@ Extra fields:
   component names, and similarly each of CorpName, FamName, Name, and
   PersName stores a name assembled from its component parts.
 
+
+Model validation:
+
+* Validation of certain fields based on the XML attribute requirements
+  (NMTOKEN, etc) is handled.
+
+* XML content model validation is handled partly through the model
+  definitions and occasionally with clean methods. The latter are used
+  in cases where at least one of a number of options is required, but
+  cannot be a full solution where many-to-many relationships are
+  involved - an instance needs to exist before it can be associated
+  through a ManyToManyField, and so validation will not be run on
+  initial creation.
+
+* The content model of certain XML elements is not yet validated, but
+  should be handled in future through specific validators that use
+  snippets of the EAD3 XSD to perform XML validation.
+
 """
 from copy import deepcopy
 from xml.sax.saxutils import escape
 
+from django.core.exceptions import ValidationError
 from django.db import models
 
 from lxml import etree
@@ -470,7 +489,14 @@ class LanguageMixin(models.Model):
 class NameMixin(models.Model):
     """Provides an extra field and save method for name models that stores
     a name assembled from its parts."""
-    assembled_name = models.TextField()
+    assembled_name = models.TextField(blank=True)
+
+    def clean(self):
+        if self.pk is None:
+            return
+        if self.part_set.count() == 0:
+            raise ValidationError('A {} must have at least one part.'.format(
+                self._meta.object_name))
 
     def save(self, *args, **kwargs):
         self.assembled_name = ''.join(
@@ -1112,7 +1138,7 @@ class Container(CommonMixin, EncodingAnalogMixin, LabelMixin, LangScriptMixin,
     container = models.TextField()
     containerid = models.CharField(blank=True, max_length=128)
     parents = models.ManyToManyField(
-        'self', related_name='child_containers', symmetrical=False)
+        'self', blank=True, related_name='child_containers', symmetrical=False)
 
 
 @reversion.register()
@@ -1148,12 +1174,7 @@ class ConventionDeclaration(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
 @reversion.register(follow=['part_set'])
 class CorpName(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
                LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 corpname.
-
-    EAD3 specifies that a corpname must have at least one part child;
-    validation of this within Django must be done within a form.
-
-    """
+    """Corresponds to EAD3 corpname."""
     pass
 
 
@@ -1241,12 +1262,7 @@ class EventDescription(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
 @reversion.register(follow=['part_set'])
 class FamName(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
               LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 famname.
-
-    EAD3 specifies that a famname must have at least one part child;
-    validation of this within Django must be done within a form.
-
-    """
+    """Corresponds to EAD3 famname."""
     pass
 
 
@@ -1491,12 +1507,7 @@ class MaterialSpec(CommonMixin, EncodingAnalogMixin, LabelMixin,
 @reversion.register(follow=['part_set'])
 class Name(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
            LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 name.
-
-    EAD3 specifies that a name must have at least one part child;
-    validation of this within Django must be done within a form.
-
-    """
+    """Corresponds to EAD3 name."""
     pass
 
 
@@ -1532,10 +1543,23 @@ class Origination(CommonMixin, EncodingAnalogMixin, LabelMixin,
                   LangScriptMixin, LocalTypeMixin):
     """Corresponds to EAD3 origination."""
     did = models.ForeignKey(EAD, on_delete=models.CASCADE)
-    corpnames = models.ManyToManyField(CorpName)
-    famnames = models.ManyToManyField(FamName)
-    names = models.ManyToManyField(Name)
-    persnames = models.ManyToManyField('PersName')
+    corpnames = models.ManyToManyField(CorpName, blank=True)
+    famnames = models.ManyToManyField(FamName, blank=True)
+    names = models.ManyToManyField(Name, blank=True)
+    persnames = models.ManyToManyField('PersName', blank=True)
+
+    def clean(self):
+        # There must be at least one CorpName, FamnName, Name, or
+        # PersName associated with an Origination. Since these are
+        # many to many fields, we must be able to create an
+        # Origination instance before populating them, so do not run
+        # this validation if there is no primary key.
+        if self.pk is None:
+            return
+        if self.corpnames.count() + self.famnames.count() + \
+                self.names.count() + self.persnames.count() == 0:
+            raise ValidationError(
+                'Origination must reference at least one named entity.')
 
 
 @reversion.register()
@@ -1565,12 +1589,7 @@ class OtherRecordID(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
 @reversion.register(follow=['part_set'])
 class PersName(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
                LocalTypeMixin, NormalMixin, RelatorMixin, NameMixin):
-    """Corresponds to EAD3 pername.
-
-    EAD3 specifies that a persname must have at least one part child;
-    validation of this within Django must be done within a form.
-
-    """
+    """Corresponds to EAD3 persname."""
     pass
 
 
@@ -1631,7 +1650,7 @@ class PhysLoc(CommonMixin, EncodingAnalogMixin, LabelMixin, LangScriptMixin,
     """Corresponds to EAD3 physloc."""
     did = models.ForeignKey(EAD, on_delete=models.CASCADE)
     parents = models.ManyToManyField(
-        'self', related_name='child_physlocs', symmetrical=False)
+        'self', blank=True, related_name='child_physlocs', symmetrical=False)
     physloc = models.TextField()
 
 
@@ -1736,10 +1755,23 @@ class Repository(CommonMixin, EncodingAnalogMixin, LabelMixin, LangScriptMixin,
     address_lang = deepcopy(lang)
     address_script = deepcopy(script)
     assembled_name = models.TextField(blank=True)
-    corpnames = models.ManyToManyField(CorpName)
-    famnames = models.ManyToManyField(FamName)
-    names = models.ManyToManyField(Name)
-    persnames = models.ManyToManyField(PersName)
+    corpnames = models.ManyToManyField(CorpName, blank=True)
+    famnames = models.ManyToManyField(FamName, blank=True)
+    names = models.ManyToManyField(Name, blank=True)
+    persnames = models.ManyToManyField(PersName, blank=True)
+
+    def clean(self):
+        # There must be at least one CorpName, FamnName, Name, or
+        # PersName associated with a Repository. Since these are
+        # many to many fields, we must be able to create an
+        # Repository instance before populating them, so do not run
+        # this validation if there is no primary key.
+        if self.pk is None:
+            return
+        if self.corpnames.all().count() + self.famnames.all().count() + \
+           self.names.all().count() + self.persnames.all().count() == 0:
+            raise ValidationError(
+                'Repository must reference at least one named entity.')
 
     def save(self, *args, **kwargs):
         # Populate the assembled_name field from data drawn from all
